package at.ac.uibk.gitsearch.web.rest;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Optional;

import at.ac.uibk.gitsearch.config.ApplicationProperties;
import at.ac.uibk.gitsearch.es.model.ArtemisExerciseInfo;
import at.ac.uibk.gitsearch.service.ArtemisImportError;
import at.ac.uibk.gitsearch.service.ExerciseService;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.gitlab4j.api.GitLabApiException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import at.ac.uibk.gitsearch.service.GitlabService;
import at.ac.uibk.gitsearch.web.util.HeaderUtil;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.core.Response;

import static javax.ws.rs.core.Response.ok;

/**
 * this resource should give access to various aspects of an exercise or course
 * @author Michael Breu
 *
 */
@RestController
@RequestMapping("/api")
public class ExerciseResource {

    private final Logger log = LoggerFactory.getLogger(ExerciseResource.class);

    @Value("${application.registeredConnectorsCallBackURL}")
    String baseApiUrl;

    @Autowired
    private GitlabService gitLabService;

    @Autowired
    private ExerciseService exerciseService;

    @Value("${jhipster.clientApp.name}")
    private String applicationName;


    /**
     * GET readMe : get problem statement.
     *
     * @param basketToken the basket
     * @return the ResponseEntity with status 200 (OK) and with body the exercise, or with status 404 (Not Found)
     */
    @GetMapping("/exercise/{exerciseId}")
    public ResponseEntity<Resource> getRepositoryFile(@PathVariable("exerciseId") String  exerciseId, @RequestParam("filePath") String filePath) throws IOException {
        InputStream repositoryStream = null;
        try {
            repositoryStream = gitLabService.getRepositoryFile(exerciseId, filePath);
        } catch (IOException | GitLabApiException e) {
            log.error("Cannot read file for exercise {} and path {}", exerciseId, filePath, e);
            return ResponseEntity.notFound().headers(HeaderUtil.createFailureAlert(applicationName, true, "RepositoryFile", "not found",
                    "There was an error on the server and the resource file could not be created.")).build();        }

        InputStreamResource resource = new InputStreamResource(repositoryStream);

        return ResponseEntity.ok().contentType(MediaType.APPLICATION_OCTET_STREAM).header("filename", filePath).body(resource);
    }

    /**
     * GET /exercise/import : Endpoint listening for request to import exercises.
     * Ex. ArTEMiS calls this endpoint and provides an URL where the exercise can be retrieved
     * @param exerciseUrl URL where the exercise can be retrieved by GitSearch
     * @param apiKey Key used to validate the request
     * @param request Used to retrieve the baseUrl of the request
     * @return URI to the client page containing the exercise import form
     * @throws ArtemisImportError
     * @throws MalformedURLException
     * @throws URISyntaxException
     */
    @GetMapping("/exercise/import")
    public ResponseEntity<URI> importExerciseFromArtemis(@RequestParam("exerciseUrl") String exerciseUrl,
                                              @RequestParam("apiKey") Optional<String> apiKey,
                                              HttpServletRequest request) throws ArtemisImportError, MalformedURLException, URISyntaxException {
        if (apiKey.isPresent() && exerciseService.validate(apiKey.get())) {
            exerciseService.importExerciseFromArtemis(exerciseUrl);
        } else {
            return ResponseEntity.status(401).body(null);
        }

        String baseUrl = getBaseUrl(request);
        String exerciseToken = exerciseUrl.split("/")[exerciseUrl.split("/").length - 1];

        return ResponseEntity.ok(new URL(baseUrl + "/import/" + exerciseToken).toURI());
    }

    /**
     * GET /exercise/imported-exercise-info/{exerciseToken} : Used to retrieve the exercise's metadata
     * of a given exerciseToken
     * @param exerciseToken of the exercise to retrieve
     * @return Object containing all metadata of an ArTEMiS exercise
     * @throws IOException
     */
    @GetMapping("/exercise/imported-exercise-info/{exerciseToken}")
    public ResponseEntity<ArtemisExerciseInfo> getImportedExerciseInfo(@PathVariable("exerciseToken") String exerciseToken) throws IOException {
        return  ResponseEntity.ok(exerciseService.getArtemisExerciseInfo(exerciseToken));
    }

    /**
     * POST /exercise/import-exercise/{exerciseToken}/{gitlabGroupId} : Endpoint requesting the actual
     * import to GitSearch/GitLab.
     * @param exerciseInfo Exercise's metadata object
     * @param exerciseToken of the exercise to import
     * @param gitlabGroupId of the Gitlab group in which the exercise repository should be created
     * @return Response with redirect if the import was successful
     * @throws GitLabApiException
     * @throws GitAPIException
     * @throws IOException
     */
    @PostMapping("/exercise/import-exercise/{exerciseToken}/{gitlabGroupId}")
    public Response importExercise(@RequestBody() ArtemisExerciseInfo exerciseInfo,
                                   @PathVariable("exerciseToken") String exerciseToken,
                                   @PathVariable("gitlabGroupId") Integer gitlabGroupId) throws GitLabApiException, GitAPIException, IOException {
        exerciseService.importExercise(exerciseInfo, exerciseToken, gitlabGroupId);
        return Response.ok().build();
    }

    private static String getBaseUrl(HttpServletRequest request) {
        String scheme = request.getScheme() + "://";
        String serverName = request.getServerName();
        String serverPort = (request.getServerPort() == 80) ? "" : ":" + request.getServerPort();
        String contextPath = request.getContextPath();
        return scheme + serverName + serverPort + contextPath;
    }
}
