package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.domain.SavedSearches;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.repository.SavedSearchesRepository;
import at.ac.uibk.gitsearch.repository.search.SavedSearchesSearchRepository;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;
import at.ac.uibk.gitsearch.service.SavedSearchesService;
import at.ac.uibk.gitsearch.service.dto.SavedSearchesDTO;
import at.ac.uibk.gitsearch.service.mapper.SavedSearchesMapper;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;
import javax.persistence.EntityManager;
import java.util.Collections;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.elasticsearch.index.query.QueryBuilders.queryStringQuery;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

/**
 * Integration tests for the {@link SavedSearchesResource} REST controller.
 */
@SpringBootTest(classes = GitsearchApp.class)
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
public class SavedSearchesResourceIT {

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_JSON_QUERY = "AAAAAAAAAA";
    private static final String UPDATED_JSON_QUERY = "BBBBBBBBBB";

    @Autowired
    private SavedSearchesRepository savedSearchesRepository;

    @Autowired
    private SavedSearchesMapper savedSearchesMapper;

    @Autowired
    private SavedSearchesService savedSearchesService;

    /**
     * This repository is mocked in the at.ac.uibk.gitsearch.repository.search test package.
     *
     * @see at.ac.uibk.gitsearch.repository.search.SavedSearchesSearchRepositoryMockConfiguration
     */
    @Autowired
    private SavedSearchesSearchRepository mockSavedSearchesSearchRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restSavedSearchesMockMvc;

    private SavedSearches savedSearches;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SavedSearches createEntity(EntityManager em) {
        SavedSearches savedSearches = new SavedSearches()
            .name(DEFAULT_NAME)
            .jsonQuery(DEFAULT_JSON_QUERY);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();
        savedSearches.setUserId(user);
        return savedSearches;
    }
    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SavedSearches createUpdatedEntity(EntityManager em) {
        SavedSearches savedSearches = new SavedSearches()
            .name(UPDATED_NAME)
            .jsonQuery(UPDATED_JSON_QUERY);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();
        savedSearches.setUserId(user);
        return savedSearches;
    }

    @BeforeEach
    public void initTest() {
        savedSearches = createEntity(em);
    }

    @Test
    @Transactional
    public void createSavedSearches() throws Exception {
        int databaseSizeBeforeCreate = savedSearchesRepository.findAll().size();
        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);
        restSavedSearchesMockMvc.perform(post("/api/saved-searches")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO)))
            .andExpect(status().isCreated());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeCreate + 1);
        SavedSearches testSavedSearches = savedSearchesList.get(savedSearchesList.size() - 1);
        assertThat(testSavedSearches.getName()).isEqualTo(DEFAULT_NAME);
        assertThat(testSavedSearches.getJsonQuery()).isEqualTo(DEFAULT_JSON_QUERY);

        // Validate the SavedSearches in Elasticsearch
        verify(mockSavedSearchesSearchRepository, times(1)).save(testSavedSearches);
    }

    @Test
    @Transactional
    public void createSavedSearchesWithExistingId() throws Exception {
        int databaseSizeBeforeCreate = savedSearchesRepository.findAll().size();

        // Create the SavedSearches with an existing ID
        savedSearches.setId(1L);
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        // An entity with an existing ID cannot be created, so this API call must fail
        restSavedSearchesMockMvc.perform(post("/api/saved-searches")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO)))
            .andExpect(status().isBadRequest());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeCreate);

        // Validate the SavedSearches in Elasticsearch
        verify(mockSavedSearchesSearchRepository, times(0)).save(savedSearches);
    }


    @Test
    @Transactional
    public void checkNameIsRequired() throws Exception {
        int databaseSizeBeforeTest = savedSearchesRepository.findAll().size();
        // set the field null
        savedSearches.setName(null);

        // Create the SavedSearches, which fails.
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);


        restSavedSearchesMockMvc.perform(post("/api/saved-searches")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO)))
            .andExpect(status().isBadRequest());

        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    public void checkJsonQueryIsRequired() throws Exception {
        int databaseSizeBeforeTest = savedSearchesRepository.findAll().size();
        // set the field null
        savedSearches.setJsonQuery(null);

        // Create the SavedSearches, which fails.
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);


        restSavedSearchesMockMvc.perform(post("/api/saved-searches")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO)))
            .andExpect(status().isBadRequest());

        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    public void getAllSavedSearches() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        // Get all the savedSearchesList
        restSavedSearchesMockMvc.perform(get("/api/saved-searches?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(savedSearches.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].jsonQuery").value(hasItem(DEFAULT_JSON_QUERY)));
    }
    
    @Test
    @Transactional
    public void getSavedSearches() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        // Get the savedSearches
        restSavedSearchesMockMvc.perform(get("/api/saved-searches/{id}", savedSearches.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(savedSearches.getId().intValue()))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.jsonQuery").value(DEFAULT_JSON_QUERY));
    }
    @Test
    @Transactional
    public void getNonExistingSavedSearches() throws Exception {
        // Get the savedSearches
        restSavedSearchesMockMvc.perform(get("/api/saved-searches/{id}", Long.MAX_VALUE))
            .andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    public void updateSavedSearches() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();

        // Update the savedSearches
        SavedSearches updatedSavedSearches = savedSearchesRepository.findById(savedSearches.getId()).get();
        // Disconnect from session so that the updates on updatedSavedSearches are not directly saved in db
        em.detach(updatedSavedSearches);
        updatedSavedSearches
            .name(UPDATED_NAME)
            .jsonQuery(UPDATED_JSON_QUERY);
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(updatedSavedSearches);

        restSavedSearchesMockMvc.perform(put("/api/saved-searches")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO)))
            .andExpect(status().isOk());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
        SavedSearches testSavedSearches = savedSearchesList.get(savedSearchesList.size() - 1);
        assertThat(testSavedSearches.getName()).isEqualTo(UPDATED_NAME);
        assertThat(testSavedSearches.getJsonQuery()).isEqualTo(UPDATED_JSON_QUERY);

        // Validate the SavedSearches in Elasticsearch
        verify(mockSavedSearchesSearchRepository, times(1)).save(testSavedSearches);
    }

    @Test
    @Transactional
    public void updateNonExistingSavedSearches() throws Exception {
        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();

        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSavedSearchesMockMvc.perform(put("/api/saved-searches")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO)))
            .andExpect(status().isBadRequest());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);

        // Validate the SavedSearches in Elasticsearch
        verify(mockSavedSearchesSearchRepository, times(0)).save(savedSearches);
    }

    @Test
    @Transactional
    public void deleteSavedSearches() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        int databaseSizeBeforeDelete = savedSearchesRepository.findAll().size();

        // Delete the savedSearches
        restSavedSearchesMockMvc.perform(delete("/api/saved-searches/{id}", savedSearches.getId())
           	.with(csrf().asHeader())
            .accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeDelete - 1);

        // Validate the SavedSearches in Elasticsearch
        verify(mockSavedSearchesSearchRepository, times(1)).deleteById(savedSearches.getId());
    }

    @Test
    @Transactional
    public void searchSavedSearches() throws Exception {
        // Configure the mock search repository
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);
        when(mockSavedSearchesSearchRepository.search(queryStringQuery("id:" + savedSearches.getId()), PageRequest.of(0, 20)))
            .thenReturn(new PageImpl<>(Collections.singletonList(savedSearches), PageRequest.of(0, 1), 1));

        // Search the savedSearches
        restSavedSearchesMockMvc.perform(get("/api/_search/saved-searches?query=id:" + savedSearches.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(savedSearches.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].jsonQuery").value(hasItem(DEFAULT_JSON_QUERY)));
    }
}
