package at.ac.uibk.gitsearch.web.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.List;

import javax.persistence.EntityManager;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.UserWatchList;
import at.ac.uibk.gitsearch.repository.UserWatchListRepository;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;
import at.ac.uibk.gitsearch.service.UserWatchListService;
import at.ac.uibk.gitsearch.service.dto.UserWatchListDTO;
import at.ac.uibk.gitsearch.service.mapper.UserWatchListMapper;

/**
 * Integration tests for the {@link UserWatchListResource} REST controller.
 */
@SpringBootTest(classes = GitsearchApp.class)
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
public class UserWatchListResourceIT {

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    @Autowired
    private UserWatchListRepository userWatchListRepository;

    @Autowired
    private UserWatchListMapper userWatchListMapper;

    @Autowired
    private UserWatchListService userWatchListService;


    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restUserWatchListMockMvc;

    private UserWatchList userWatchList;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static UserWatchList createEntity(EntityManager em) {
        UserWatchList userWatchList = new UserWatchList()
            .name(DEFAULT_NAME);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();
        userWatchList.setUserId(user);
        return userWatchList;
    }
    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static UserWatchList createUpdatedEntity(EntityManager em) {
        UserWatchList userWatchList = new UserWatchList()
            .name(UPDATED_NAME);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();
        userWatchList.setUserId(user);
        return userWatchList;
    }

    @BeforeEach
    public void initTest() {
        userWatchList = createEntity(em);
    }

    @Test
    @Transactional
    public void createUserWatchList() throws Exception {
        int databaseSizeBeforeCreate = userWatchListRepository.findAll().size();
        // Create the UserWatchList
        UserWatchListDTO userWatchListDTO = userWatchListMapper.toDto(userWatchList);
        restUserWatchListMockMvc.perform(post("/api/user-watch-lists")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(userWatchListDTO)))
            .andExpect(status().isCreated());

        // Validate the UserWatchList in the database
        List<UserWatchList> userWatchListList = userWatchListRepository.findAll();
        assertThat(userWatchListList).hasSize(databaseSizeBeforeCreate + 1);
        UserWatchList testUserWatchList = userWatchListList.get(userWatchListList.size() - 1);
        assertThat(testUserWatchList.getName()).isEqualTo(DEFAULT_NAME);

    }

    @Test
    @Transactional
    public void createUserWatchListWithExistingId() throws Exception {
        int databaseSizeBeforeCreate = userWatchListRepository.findAll().size();

        // Create the UserWatchList with an existing ID
        userWatchList.setId(1L);
        UserWatchListDTO userWatchListDTO = userWatchListMapper.toDto(userWatchList);

        // An entity with an existing ID cannot be created, so this API call must fail
        restUserWatchListMockMvc.perform(post("/api/user-watch-lists")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(userWatchListDTO)))
            .andExpect(status().isBadRequest());

        // Validate the UserWatchList in the database
        List<UserWatchList> userWatchListList = userWatchListRepository.findAll();
        assertThat(userWatchListList).hasSize(databaseSizeBeforeCreate);

    }


    @Test
    @Transactional
    public void checkNameIsRequired() throws Exception {
        int databaseSizeBeforeTest = userWatchListRepository.findAll().size();
        // set the field null
        userWatchList.setName(null);

        // Create the UserWatchList, which fails.
        UserWatchListDTO userWatchListDTO = userWatchListMapper.toDto(userWatchList);


        restUserWatchListMockMvc.perform(post("/api/user-watch-lists")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(userWatchListDTO)))
            .andExpect(status().isBadRequest());

        List<UserWatchList> userWatchListList = userWatchListRepository.findAll();
        assertThat(userWatchListList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    public void getAllUserWatchLists() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        // Get all the userWatchListList
        restUserWatchListMockMvc.perform(get("/api/user-watch-lists?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(userWatchList.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)));
    }
    
    @Test
    @Transactional
    public void getUserWatchList() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        // Get the userWatchList
        restUserWatchListMockMvc.perform(get("/api/user-watch-lists/{id}", userWatchList.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(userWatchList.getId().intValue()))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME));
    }


    @Test
    @Transactional
    public void getUserWatchListsByIdFiltering() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        Long id = userWatchList.getId();

        defaultUserWatchListShouldBeFound("id.equals=" + id);
        defaultUserWatchListShouldNotBeFound("id.notEquals=" + id);

        defaultUserWatchListShouldBeFound("id.greaterThanOrEqual=" + id);
        defaultUserWatchListShouldNotBeFound("id.greaterThan=" + id);

        defaultUserWatchListShouldBeFound("id.lessThanOrEqual=" + id);
        defaultUserWatchListShouldNotBeFound("id.lessThan=" + id);
    }


    @Test
    @Transactional
    public void getAllUserWatchListsByNameIsEqualToSomething() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        // Get all the userWatchListList where name equals to DEFAULT_NAME
        defaultUserWatchListShouldBeFound("name.equals=" + DEFAULT_NAME);

        // Get all the userWatchListList where name equals to UPDATED_NAME
        defaultUserWatchListShouldNotBeFound("name.equals=" + UPDATED_NAME);
    }

    @Test
    @Transactional
    public void getAllUserWatchListsByNameIsNotEqualToSomething() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        // Get all the userWatchListList where name not equals to DEFAULT_NAME
        defaultUserWatchListShouldNotBeFound("name.notEquals=" + DEFAULT_NAME);

        // Get all the userWatchListList where name not equals to UPDATED_NAME
        defaultUserWatchListShouldBeFound("name.notEquals=" + UPDATED_NAME);
    }

    @Test
    @Transactional
    public void getAllUserWatchListsByNameIsInShouldWork() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        // Get all the userWatchListList where name in DEFAULT_NAME or UPDATED_NAME
        defaultUserWatchListShouldBeFound("name.in=" + DEFAULT_NAME + "," + UPDATED_NAME);

        // Get all the userWatchListList where name equals to UPDATED_NAME
        defaultUserWatchListShouldNotBeFound("name.in=" + UPDATED_NAME);
    }

    @Test
    @Transactional
    public void getAllUserWatchListsByNameIsNullOrNotNull() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        // Get all the userWatchListList where name is not null
        defaultUserWatchListShouldBeFound("name.specified=true");

        // Get all the userWatchListList where name is null
        defaultUserWatchListShouldNotBeFound("name.specified=false");
    }
                @Test
    @Transactional
    public void getAllUserWatchListsByNameContainsSomething() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        // Get all the userWatchListList where name contains DEFAULT_NAME
        defaultUserWatchListShouldBeFound("name.contains=" + DEFAULT_NAME);

        // Get all the userWatchListList where name contains UPDATED_NAME
        defaultUserWatchListShouldNotBeFound("name.contains=" + UPDATED_NAME);
    }

    @Test
    @Transactional
    public void getAllUserWatchListsByNameNotContainsSomething() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        // Get all the userWatchListList where name does not contain DEFAULT_NAME
        defaultUserWatchListShouldNotBeFound("name.doesNotContain=" + DEFAULT_NAME);

        // Get all the userWatchListList where name does not contain UPDATED_NAME
        defaultUserWatchListShouldBeFound("name.doesNotContain=" + UPDATED_NAME);
    }


    @Test
    @Transactional
    public void getAllUserWatchListsByUserIdIsEqualToSomething() throws Exception {
        // Get already existing entity
        User userId = userWatchList.getUserId();
        userWatchListRepository.saveAndFlush(userWatchList);
        Long userIdId = userId.getId();

        // Get all the userWatchListList where userId equals to userIdId
        defaultUserWatchListShouldBeFound("userIdId.equals=" + userIdId);

        // Get all the userWatchListList where userId equals to userIdId + 1
        defaultUserWatchListShouldNotBeFound("userIdId.equals=" + (userIdId + 1));
    }

    /**
     * Executes the search, and checks that the default entity is returned.
     */
    private void defaultUserWatchListShouldBeFound(String filter) throws Exception {
        restUserWatchListMockMvc.perform(get("/api/user-watch-lists?sort=id,desc&" + filter))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(userWatchList.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)));

        // Check, that the count call also returns 1
        restUserWatchListMockMvc.perform(get("/api/user-watch-lists/count?sort=id,desc&" + filter))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(content().string("1"));
    }

    /**
     * Executes the search, and checks that the default entity is not returned.
     */
    private void defaultUserWatchListShouldNotBeFound(String filter) throws Exception {
        restUserWatchListMockMvc.perform(get("/api/user-watch-lists?sort=id,desc&" + filter))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$").isArray())
            .andExpect(jsonPath("$").isEmpty());

        // Check, that the count call also returns 0
        restUserWatchListMockMvc.perform(get("/api/user-watch-lists/count?sort=id,desc&" + filter))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(content().string("0"));
    }

    @Test
    @Transactional
    public void getNonExistingUserWatchList() throws Exception {
        // Get the userWatchList
        restUserWatchListMockMvc.perform(get("/api/user-watch-lists/{id}", Long.MAX_VALUE))
            .andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    public void updateUserWatchList() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        int databaseSizeBeforeUpdate = userWatchListRepository.findAll().size();

        // Update the userWatchList
        UserWatchList updatedUserWatchList = userWatchListRepository.findById(userWatchList.getId()).get();
        // Disconnect from session so that the updates on updatedUserWatchList are not directly saved in db
        em.detach(updatedUserWatchList);
        updatedUserWatchList
            .name(UPDATED_NAME);
        UserWatchListDTO userWatchListDTO = userWatchListMapper.toDto(updatedUserWatchList);

        restUserWatchListMockMvc.perform(put("/api/user-watch-lists")
           	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(userWatchListDTO)))
            .andExpect(status().isOk());

        // Validate the UserWatchList in the database
        List<UserWatchList> userWatchListList = userWatchListRepository.findAll();
        assertThat(userWatchListList).hasSize(databaseSizeBeforeUpdate);
        UserWatchList testUserWatchList = userWatchListList.get(userWatchListList.size() - 1);
        assertThat(testUserWatchList.getName()).isEqualTo(UPDATED_NAME);

    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.USER)
    public void updateNonExistingUserWatchList() throws Exception {
        int databaseSizeBeforeUpdate = userWatchListRepository.findAll().size();

        // Create the UserWatchList
        UserWatchListDTO userWatchListDTO = userWatchListMapper.toDto(userWatchList);
        userWatchListDTO.setId(Long.MAX_VALUE);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restUserWatchListMockMvc.perform(put("/api/user-watch-lists")
          	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(userWatchListDTO)))
            .andExpect(status().isBadRequest());

        // Validate the UserWatchList in the database
        List<UserWatchList> userWatchListList = userWatchListRepository.findAll();
        assertThat(userWatchListList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    public void updatChangeUserWatchListWithoutId() throws Exception {
        int databaseSizeBeforeUpdate = userWatchListRepository.findAll().size();

        // Create the UserWatchList
        UserWatchListDTO userWatchListDTO = userWatchListMapper.toDto(userWatchList);
        userWatchListDTO.setId(null);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restUserWatchListMockMvc.perform(put("/api/user-watch-lists")
          	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(userWatchListDTO)))
            .andExpect(status().isBadRequest());

        // Validate the UserWatchList in the database
        List<UserWatchList> userWatchListList = userWatchListRepository.findAll();
        assertThat(userWatchListList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    public void deleteUserWatchList() throws Exception {
        // Initialize the database
        userWatchListRepository.saveAndFlush(userWatchList);

        int databaseSizeBeforeDelete = userWatchListRepository.findAll().size();

        // Delete the userWatchList
        restUserWatchListMockMvc.perform(delete("/api/user-watch-lists/{id}", userWatchList.getId())
           	.with(csrf().asHeader())
            .accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<UserWatchList> userWatchListList = userWatchListRepository.findAll();
        assertThat(userWatchListList).hasSize(databaseSizeBeforeDelete - 1);

    }
}
